// MfcGlView.cpp : implementation of the CMfcGlView class
//

#include "stdafx.h"
#include "MfcGl.h"

#include "MfcGlDoc.h"
#include "MfcGlView.h"


#include "glcode.h"
#include "defines.h"

#include "light.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CMfcGlView

IMPLEMENT_DYNCREATE(CMfcGlView, CView)

BEGIN_MESSAGE_MAP(CMfcGlView, CView)
	//{{AFX_MSG_MAP(CMfcGlView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMfcGlView construction/destruction

CMfcGlView::CMfcGlView()
{
	// TODO: add construction code here
   m_pDC=NULL;
      
   lastTime=CTime::GetCurrentTime();
}

CMfcGlView::~CMfcGlView()
{
}

BOOL CMfcGlView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	//Anyadimos los estilos de ventana que requiere OPENGL antes
	//de crear la ventana
	//CSOWNDC sirve para que la ventana pueda tener su propio contexto de 
	//dispositivo privado
	cs.style|=(WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CS_OWNDC);


	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMfcGlView drawing

void CMfcGlView::OnDraw(CDC* pDC)
{
	CMfcGlDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

	
	//MICODIGO
	HDC m_hDC=pDC->GetSafeHdc();
	//Activa el contexto de generacion
	wglMakeCurrent(m_hDC,m_hRC);
	//Llama al codigo OPENGL externo
	redibujar_escena();
	//Intercambia la escena al frente
	SwapBuffers(m_hDC);

	//Permite coexistencia de otros contextos de generacion
//	wglMakeCurrent(m_hDC,NULL);
	
	
}

/////////////////////////////////////////////////////////////////////////////
// CMfcGlView diagnostics

#ifdef _DEBUG
void CMfcGlView::AssertValid() const
{
	CView::AssertValid();
}

void CMfcGlView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMfcGlDoc* CMfcGlView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMfcGlDoc)));
	return (CMfcGlDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMfcGlView message handlers

//INMA: nuevos mensajes anyadidos con el ClassWizard, para poder
//actuar frente a los eventos de creacion y destruccion de ventana

//Dentro de OnCreate establecemos el formato de pixel
int CMfcGlView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	//MICODIGO
	int nPixelFormat;			//Indice del formato de pixel

	m_pDC = new CClientDC(this);
	HDC m_hDC=m_pDC->GetSafeHdc();

	static PIXELFORMATDESCRIPTOR pfd={
		sizeof(PIXELFORMATDESCRIPTOR),	//Tamanyo de esta estructura
		1,								//Version de esta estructura
		PFD_DRAW_TO_WINDOW|			//Dibuja en la ventana(no en un mapa)
		PFD_SUPPORT_OPENGL|			//Soporte de llamada OPENGL en la ventana
		PFD_DOUBLEBUFFER,			//Modo de doble buffer
		PFD_TYPE_RGBA,				//Modo de color RGBA
		24,							//Color de 24 bits
		0,0,0,0,0,0,				//Bits de color ignorados
		0,0,							
		0,0,0,0,0,
		32,							//Tamao del buffer de profundidad 32 bits
		0,
		0,
		PFD_MAIN_PLANE,				//Dibuja en el plano principal
		0,							
		0,0,0};
	//Elegimos el formato de pixel que mejor encaje con el descrito en pfd
	if ( (nPixelFormat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0 )
    {
        MessageBox("ChoosePixelFormat failed");
        return FALSE;
    }

	//Activamos el formato de pixel para el contexto de dispositivo
	if (SetPixelFormat(m_pDC->GetSafeHdc(), nPixelFormat, &pfd) == FALSE)
    {
        MessageBox("SetPixelFormat failed");
        return FALSE;
    }
	
	//Creamos el contexto de generacion
	m_hRC=wglCreateContext(m_hDC);

	//Activamos el contexto de generacion y ejecutamos la inicializacion
	wglMakeCurrent(m_hDC,m_hRC);

	//LLamamos a nuestra funcion que inicializa el codigo de opengl
	inicializacion_gl();


	return 0;
}

//Destruccion de la ventana, borramos el contexto de generacion y liberamos
//el contexto de dispositivo
void CMfcGlView::OnDestroy() 
{

	wglDeleteContext(m_hRC);

	if (m_pDC)
        delete m_pDC;
	CView::OnDestroy();
	
	
}
/************************************************
Funcion que responde al evento de cambio de tamao
de la ventana.
****************************************************/
void CMfcGlView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	//LLamamos a nuestra funcin que actualiza el
	//viewport de opengl cuando cambia el tamao de la ventana
	cambia_tamanyo(cx,cy);
	InvalidateRect(NULL,TRUE);
	
}

//Sobrecargamos para evitar la limpieza del fondo
//cada vez que redibujemos la ventana
BOOL CMfcGlView::OnEraseBkgnd(CDC* pDC) 
{
	
//	return CView::OnEraseBkgnd(pDC);
	return FALSE;
}

/****************************************************
Funciones que responden a los eventos de la ventana de
ratn y de teclado.
**********************************************************/
void CMfcGlView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CView::OnLButtonDown(nFlags, point);

	//ZOOM HACIA ATRAS
	if(mode_act==MODE_ZOOM)
	{
		GLPosView(0, 0, -1.0f);
		
	}
	InvalidateRect(NULL,TRUE);//Forzamos redibujado.
	
}


void CMfcGlView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	
	CView::OnRButtonDown(nFlags, point);
	
	//ZOOM HACIA DELANTE
	if(mode_act==MODE_ZOOM)
	{
		GLPosView(0, 0, 1.0f);
	}
	InvalidateRect(NULL,TRUE);//Forzamos redibujado.
}

void CMfcGlView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CView::OnMouseMove(nFlags, point);

	//ROTAMOS LA ESCENA SI SE MUEVE EL RATON CON LA TECLA
	//DERECHA PULSADA Y LA TECLA ALT.
	switch(mode_act)
	{
	case MODE_MOVEMENT:
		if((nFlags & MK_SHIFT)&& (nFlags & MK_LBUTTON))//Modo rotacion
		{
			rotationy = 360.0f * (((2.0f/wide)*point.x)-1.0f);
			rotationx = 360.0f * (((2.0f/height)*point.y)-1.0f);	
		}
			
	break;
	}

	InvalidateRect(NULL,TRUE);//Forzamos redibujado.
}

void CMfcGlView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	
   CView::OnKeyDown(nChar, nRepCnt, nFlags);

   //Mi codigo
   switch(nChar)
	{
		case 'a':
		case 'A':
			if(!animacionActivada)
			{
				this->SetTimer(3,50,NULL); //Crea un timer, con identificador '3' y 50 mseg de tiempo			
			}
			else
			{
				this->KillTimer(3); //Desactiva el timer
			}
			
		break;
	}
	eventos_teclado(nChar);
	InvalidateRect(NULL,TRUE);//Forzamos redibujado.
}
/******************************************************
Mensaje WM_TIMER.
********************************************************/
void CMfcGlView::OnTimer(UINT nIDEvent) 
{

	
	CView::OnTimer(nIDEvent);

	//MICODIGO
	if(nIDEvent!=3)return;
	if(!animacionActivada)return;

	rotationy=rotationy+10;

	InvalidateRect(NULL,TRUE);//Forzamos redibujado.

}


